<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Database\Eloquent\SoftDeletes;
class Bilti extends Model
{
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the user that owns the target.
     */
    public function user()
    {
        return $this->belongsTo('App\User','supplier_id','id');
    }
    /**
     * Get the supplier that owns the target.
     */
    public function supplier()
    {
        return $this->belongsTo('App\User','delivered_to_id','id');
    }
    /***
     * Bilty Product Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */

    public function products()
    {
        return $this->hasMany('App\BiltiDetail', 'bilti_id')->with(['biltyProductTypes','products']);
    }
    /**
     * Get the Images of the bilty.
     */
    public function biltyImages() {
        return $this->hasMany('App\BiltyImage','bilty_id','id');
    }
    /***
     *  list Bilti
     *
     * @return mixed
     */
    public function listBilti(Request $request)
    {
        $data = $this::orderBy('id', 'Desc');
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('date', 'like', '%' . $request->search_name . '%');
            });
        }
        if ($request->sale_officer != '') {
            $data->where('booked_by', '=', $request->sale_officer);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update
     *
     * @return mixed
     */
    public function InsertOrUpdate(Request $request,$id=0)
    {
        $new=true;
        if($id>0){
            $bilti=$this->findOrFail($id);
            $new=false;
        }else{
        $bilti=new $this();
        }
        $bilti->bilti_no= $request->input('bilti_no');
        $bilti->po_no= $request->input('po_no');
        $bilti->supplier_id= $request->input('supplier');
        $bilti->delivered_to_id= $request->input('delivered_to');
        $bilti->bilti_date= $request->input('bilti_date');
        $bilti->bilti_received_date= $request->input('bilti_received_date');
        $bilti->status= $request->input('status');
        if($bilti->save()){
            //Bilty Products added
            if(count($request->input('products'))>0){
                $bilti_id=[];
                $stock_booking_detail_id=[];
                $all_product=BiltiDetail::select('id')->where('bilti_id',$bilti->id)->get()->toarray();
                foreach ($all_product as $product){
                    if(isset($product['id']) && !empty($product['id'])){
                        $bilti_id[]=$product['id'];
                    }
                }
                foreach ($request->input('products') as $value){
                    if(isset($value['id']) && !empty($value['id'])){
                        if (($key = array_search($value['id'], $bilti_id)) !== FALSE) {
                            unset($bilti_id[$key]);
                        }
                        $bilty_product=BiltiDetail::findOrFail($value['id']);
                    }else{
                        $bilty_product=new BiltiDetail();
                    }
                    $bilty_product->bilti_id=$bilti->id;
                    $bilty_product->product_id=$value['product'];
                    $cot=Product::where('id',$value['product'])->first();
                    if(!empty($cot)){
                        $bilty_product->quantity=$value['qty']*$cot->cotton;
                    }


//                    $bilty_product->quantity_received=$value['rcv_qty'];
                    $bilty_product->save();
                    $facial=0;
                    // Order Product Category Added
                    if($value['product']==12){
                        if(count($value['type'])>0){
                            $all_quantity=0;
                            $facial=0;
                            foreach($value['type'] as $product_category){
                                if(!empty($product_category['quantity'])){
                                    $all_quantity=$all_quantity+$product_category['quantity'];
                                    if(!empty($product_category['old_id']) && $product_category['old_id']!=0){
                                        $product_cat= BiltiDetialType::where('id',$product_category['old_id'])->first();
                                        if(empty($product_cat))
                                            $product_cat=new BiltiDetialType();
                                    }else{
                                        $product_cat=new BiltiDetialType();
                                    }
                                    $product_cat->bilti_detial_id=$bilty_product->id;
                                    $product_cat->product_category_id=$product_category['id'];
                                    if(!empty($cot)){
                                        $product_cat->quantity=$product_category['quantity']*$cot->cotton;
                                        $facial=$facial+($product_category['quantity']*$cot->cotton);
                                    }
//                                    $product_cat->quantity_received=$product_category['quantity_received'];
                                    $product_cat->save();
//                                    delivered stock type update
                                    if($id!=0){
                                        $stock_type= StockBookingDetail::where('stock_booking_id',$bilti->stock_booking_id)
                                            ->where('product_id',12)->first();
                                        $type= StockBookingDetailCategory::where('stock_booking_detail_id',$stock_type->id)
                                            ->where('product_category_id',$product_category['id'])->first();
                                        if(!empty($type)&& !empty($cot)){
                                            if($product_category['quantity']>$product_category['old_quantity']){
                                                $dif=$product_category['quantity']-$product_category['old_quantity'];
                                                $type->delivered=$type->delivered+($dif*$cot->cotton);
                                            } elseif($product_category['quantity']<$product_category['old_quantity']){
                                                $dif=$product_category['old_quantity']-$product_category['quantity'];
                                                $type->delivered=$type->delivered-($dif*$cot->cotton);
                                            }
                                            $type->save();
                                        }
//                                        if(!empty($stock_type)&& !empty($cot)){
//                                            if($facial>($value['old_qty']*$cot->cotton)){
//                                                $dif=$facial-($value['old_qty']*$cot->cotton);
//                                                $stock_type->delivered=$stock_type->delivered+($dif);
//                                            } elseif($product_category['quantity']<($value['old_qty']*$cot->cotton)){
//                                                $dif=($value['old_qty']*$cot->cotton)-$facial;
//                                                $stock_type->delivered=$stock_type->delivered-($dif);
//                                            }
//                                            $stock_type->save();
//                                        }
                                    }

                                }
                            }
                            $bilty_product->quantity=$facial;
                            $bilty_product->save();
                        }
                    }
                    //delivered stock update
                    if($id!=0){
                        $stockBooingDetail= StockBookingDetail::where('stock_booking_id',$bilti->stock_booking_id)
                            ->where('product_id',$bilty_product->product_id)->first();
                        if(!empty($stockBooingDetail)&& !empty($cot)){
                            if($value['product']==12){
                                if($facial>($value['old_qty']*$cot->cotton)){
                                    $dif=$facial-($value['old_qty']*$cot->cotton);
                                    $stockBooingDetail->delivered=$stockBooingDetail->delivered+($dif);
                                } elseif($facial<($value['old_qty']*$cot->cotton)){
                                    $dif=($value['old_qty']*$cot->cotton)-$facial;
                                    $stockBooingDetail->delivered=$stockBooingDetail->delivered-($dif);
                                }
                            }else{
                                if($facial>$value['old_qty']){
                                    $dif=$facial-$value['old_qty'];
                                    $stockBooingDetail->delivered=$stockBooingDetail->delivered+($dif*$cot->cotton);
                                } elseif($facial<$value['old_qty']){
                                    $dif=$value['old_qty']-$facial;
                                    $stockBooingDetail->delivered=$stockBooingDetail->delivered-($dif*$cot->cotton);
                                }
                            }

                            $stockBooingDetail->save();
                        }
                    }
                }
                if(count($bilti_id)>0){
                    foreach ($bilti_id as $id){
                        $old_bilti_product=BiltiDetail::findOrFail($id);
                        $old_bilti_product->delete();
                    }
                }
            }
            // Append Image
            if($new==0){
                $old_image_id=[];
                $all_product=BiltyImage::select('id')->where('bilty_id',$bilti->id)->get()->toarray();
                if(count($all_product)>0){
                    foreach ($all_product as $all){
                        if(isset($all['id']) && !empty($all['id'])){
                            $old_image_id[]=$all['id'];
                        }
                    }
                }
                if(count($request->input('old_images_id'))>0){
                    foreach ($request->input('old_images_id') as $value){
                        if (($key = array_search($value, $old_image_id)) !== FALSE) {
                            unset($old_image_id[$key]);
                        }
                    }
                }
                if(count($old_image_id)>0){
                    foreach ($old_image_id as $id){
                        $old_order_product=BiltyImage::findOrFail($id);
                        $old_order_product->delete();
                        $file = public_path() .str_replace(url(''),"",$old_order_product['image']) ;
                        if (file_exists($file)) {
                            @unlink($file);
                        }
                    }
                }
            }
            for($i=-1,$j=0;$i<12;$i++,$j++){
                if ($request->hasFile('image'.$j)) {
                    $filename = 'image-' . time() . rand(99, 199) . '.' . $request->file('image'.$j)->getClientOriginalExtension();
                    $request->file('image'.$j)->move(config('constants.UPLOAD_PATH') . 'bilty/', $filename);
                    if (isset($filename)){
                        $pro_image=new BiltyImage();
                        $pro_image->image = url('/uploads/bilty/'.$filename);
                        $pro_image->bilty_id = $bilti->id;
                        $pro_image->save();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
    public function updateStock($biltyid,$stock_id){
        $biltyProduct=BiltiDetail::where('bilti_id',$biltyid)->with('biltyProductTypes')->get();
        if(!empty($biltyProduct)){
            foreach ($biltyProduct as $bpro){
               $s_booking=StockBookingDetail::where('stock_booking_id',$stock_id) ->where('product_id',$bpro['product_id'])->first();
               if(!empty($s_booking)){
                   $s_booking->delivered=$s_booking->delivered-$bpro['quantity'];
                   $s_booking->save();
                   if($bpro['product_id']==12){
                       if(isset($bpro['biltyProductTypes']) && !empty($bpro['biltyProductTypes'])){
                           foreach ($bpro['biltyProductTypes'] as $b_type){
                               $s_booking_cat=StockBookingDetailCategory::where('stock_booking_detail_id',$s_booking->id)->where('product_category_id',$b_type['product_category_id'])->first();
                               $s_booking_cat->delivered=$s_booking_cat->delivered-$b_type['quantity'];
                               $s_booking_cat->save();
                           }
                       }
                   }
               }
            }
        }
return true;

    }
}
