<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
class PartyName extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    protected $primaryKey = 'id';
    /**
     * Get the Region that owns the City.
     */
    public function region()
    {
        return $this->belongsTo('App\Region')->with('country');
    }
    public function partyPrice()
    {
        return $this->hasMany('App\PartyRates', 'party_id','id');
    }
    public function party_ledger()
    {
        return $this->hasMany('App\UserExpenseDetail', 'party_id','id');
    }

    public function partyaccountant()
    {
        return $this->belongsTo('App\User','user_id','id');
    }
    /***
     * List Cities
     *
     * @return mixed
     */
    public function listAll(Request $request)
    {
        $data = $this::orderBy('id', 'Asc');
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('name', 'like', '%' . $request->search_name . '%');
            });
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update City
     *
     * @return mixed
     */
    public function InsertOrUpdate(Request $request,$id=0)
    {
        if($id>0){
            $city=$this->findOrFail($id);
        }else{
            $city=new $this();
            $city->opening_balance= $request->input('opening_balance');
        }

        $city->region_id= $request->input('region');
        $city->name= $request->input('name');
        $city->city= $request->input('city');

        $city->balance= $request->input('balance');
        $city->opening_balance= $request->input('opening_balance');
        $city->discount= $request->input('discount');
        if($city->save()){
            return true;
        }else{
            return false;
        }
    }
    public function refreshLedger($id=0)
    {
        $obj=$this->where('id',$id)->with(['party_ledger'=>function($query){$query->orderBy('date', 'ASC');}])->first();
        if(!empty($obj) && !empty($obj->party_ledger)){
            $opening=$obj->opening_balance;
            foreach ($obj->party_ledger as $key=>$item){
                $update=UserExpenseDetail::where('id',$item['id'])->first();
                if(!empty($update)){
                    switch ($update['type']){
                        case 'Sale':
                            $opening=$opening+$update['amount'];
                            $update->balance=$opening;
                            $update->save();
                            break;
                        case 'Recovery':
                            $opening=$opening-$update['amount'];
                            $update->balance=$opening;
                            $update->save();
                            break;
                        case 'Credit Note':
                            $opening=$opening-$update['amount'];
                            $update->balance=$opening;
                            $update->save();
                            break;
                    }
                }
                $update->save();
            }
            $obj->balance=$opening;
            $obj->save();
        }
        return true;
    }
}
