<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Database\Eloquent\SoftDeletes;
use Auth;
use Mail;
class Payment extends Model
{
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the user that owns the payment.
     */
    public function user()
    {
        return $this->belongsTo('App\User','user_id');
    }
    /**
     * Get the Shop that owns the order.
     */
    public function shop()
    {
        return $this->belongsTo('App\Shop','shop_id');
    }
    /**
     * Get the Images of the payment.
     */
    public function paymentImages() {
        return $this->hasMany('App\PaymentImage','payment_id','id');
    }

    /***
     * List Payments
     *
     * @return mixed
     */
    public function listPayments(Request $request)
    {
        $data = $this::where('agency_id',Auth::user()->agency_id)->whereIn('payment_type', ['Cash','Promise'])->orderBy('id', 'Desc');
        if ($request->search_other != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('amount', '=', $request->search_other );
            });
        }
        if ($request->sale_officer != '') {
            $data->where('user_id', '=', $request->sale_officer);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }

    // Pending Payments
    public function pendingPayments(Request $request){
        $data = $this::where('agency_id',Auth::user()->agency_id)->where('payment_type', 'Cheque')->orderBy('promise_cheque_date', 'Asc');
        if ($request->search_other != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('amount', '=', $request->search_other );
            });
        }
        if ($request->sale_officer != '') {
            $data->where('user_id', '=', $request->sale_officer);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update Payment
     *
     * @return mixed
     */
    public function InsertOrUpdatePayment(Request $request,$id=0)
    {
        $new=0;
        if($id>0){
            $payment=$this->findOrFail($id);
            $new=1;
        }else{
            $payment=new $this();
        }
        $payment->uuid= generate_uuid();
        $payment->agency_id =  Auth::user()->agency_id;
        $payment->shop_id= $request->input('shop');
        $payment->user_id= $request->input('collect_by');
        $payment->payment_type= $request->input('payment_type');
        $payment->amount= $request->input('amount');
        if($request->input('payment_type')!='Cash'){
            $payment->promise_cheque_date= $request->input('promise_cheque_date');
            $payment->cheque_type= $request->input('cheque_type');
            $payment->cheque_no= $request->input('cheque_no');
            $payment->bank_id= $request->input('bank');
        }else{
            $payment->promise_cheque_date= $request->input('promise_cheque_date');
        }
        $payment->remarks= $request->input('remarks');
        if($payment->save()){
            // Opening balance update
            if($new==1){
                if($request->input('payment_type')=='Cash') {
                    $diff = $request->input('old_amount') - $request->input('amount');
                    $shop = Shop::where('id', $request->input('shop'))->first();
                    $old_balance = $shop['balance']*-1;
                    $shop->balance = ($shop->balance) - ($diff);
                    $shop->save();
                    $payment->balance = ($payment->balance) - ($diff);
                    $payment->save();
                    // So Balance Adjustment
                    $so_balance=User::where('id',$payment->user_id)->first();
                    $so_balance->balance=($so_balance->balance) - ($diff);
                    $so_balance->save();
                    // So ledger handle
                    $soLedger= UserExpenseDetail::where('date',$payment->promise_cheque_date)
                        ->where('user_id',$payment->user_id)
                        ->first();
                    if(empty($soLedger))
                    $soLedger=new UserExpenseDetail();
                    $soLedger->user_id=$payment->user_id;
                    $soLedger->type="Recovery";
                    $soLedger->amount=$payment->amount;
                    $soLedger->balance=$so_balance->balance;
                    $soLedger->date=$payment->promise_cheque_date;
                    $soLedger->save();
                    // Send Email on order delivered
                    $new_balance =  $payment->amount;
                    $total_balance = ($shop->balance)*-1;
                    $customer_mail = $shop['email'];
                    $customer_name = $shop['name'];
                    $date = date('Y-m-d');
                    if(empty($customer_mail) || url('')=='http://demo.biocospk.com') {
                        Mail::send('email.payment_received', compact('old_balance', 'new_balance','total_balance','date','customer_name'), function($message) use ($customer_mail){
                            $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
                            $message->to('shakir130@hotmail.com')->subject('Payment Received!');
                        });
                    }else{
                        Mail::send('email.payment_received', compact('old_balance', 'new_balance','total_balance','date','customer_name'), function($message) use ($customer_mail){
                            $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
                            $message->to($customer_mail)->subject('Payment Received!');
                        });
                    }
                }
            }else{
                if($request->input('payment_type')=='Cash'){
                    $shop=Shop::where('id',$request->input('shop'))->first();
                    $old_balance = $shop['balance']*-1;
                    $shop->balance=($shop->balance)+($request->input('amount'));
                    $shop->save();
                    $payment->balance= $shop->balance;
                    $payment->save();
                    // So Balance Adjustment
                    $so_balance=User::where('id',$payment->user_id)->first();
                    $so_balance->balance=($so_balance->balance)+($request->input('amount'));
                    $so_balance->save();
                    // So ledger handle
                    $soLedger=new UserExpenseDetail();
                    $soLedger->user_id=$payment->user_id;
                    $soLedger->type="Recovery";
                    $soLedger->amount=$payment->amount;
                    $soLedger->balance=$so_balance->balance;
                    $soLedger->date=$payment->promise_cheque_date;
                    $soLedger->save();
                    // Send Email on order delivered
                    $new_balance =  $payment->amount;
                    $total_balance = ($shop->balance)*-1;
                    $customer_mail = $shop['email'];
                    $customer_name = $shop['name'];
                    $date = date('Y-m-d');


                    if(empty($customer_mail) || url('')=='http://demo.biocospk.com') {
                        Mail::send('email.payment_received', compact('old_balance', 'new_balance','total_balance','date','customer_name'), function($message) use ($customer_mail){
                            $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
                            $message->to('shakir130@hotmail.com')->subject('Payment Received!');
                        });
                    }else{
                        Mail::send('email.payment_received', compact('old_balance', 'new_balance','total_balance','date','customer_name'), function($message) use ($customer_mail){
                            $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
                            $message->to($customer_mail)->subject('Payment Received!');
                        });
                    }
                }
            }
            if($new==1){
                $old_image_id=[];
                $all_product=PaymentImage::select('id')->where('payment_id',$payment->id)->get()->toarray();
                if(count($all_product)>0){
                    foreach ($all_product as $all){
                        if(isset($all['id']) && !empty($all['id'])){
                            $old_image_id[]=$all['id'];
                        }
                    }
                }
                if(count($request->input('old_images_id'))>0){
                    foreach ($request->input('old_images_id') as $value){
                        if (($key = array_search($value, $old_image_id)) !== FALSE) {
                            unset($old_image_id[$key]);
                        }
                    }
                }
                if(count($old_image_id)>0){
                    foreach ($old_image_id as $id){
                        $old_order_product=PaymentImage::findOrFail($id);
                        $old_order_product->delete();
                        $file = public_path() .str_replace(url(''),"",$old_order_product['image']) ;
                        if (file_exists($file)) {
                            @unlink($file);
                        }
                    }
                }
            }
            for($i=-1,$j=0;$i<12;$i++,$j++){
                if ($request->hasFile('image'.$j)) {
                    $filename = 'image-' . time() . rand(99, 199) . '.' . $request->file('image'.$j)->getClientOriginalExtension();
                    $request->file('image'.$j)->move(config('constants.UPLOAD_PATH') . 'payments/', $filename);
                    if (isset($filename)){
                        $pro_image=new PaymentImage();
                        $pro_image->image = url('/uploads/payments/'.$filename);
                        $pro_image->payment_id = $payment->id;
                        $pro_image->save();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
}
